#pragma once

#include <Orange/Core/AlignedStorage.h>
#include <Orange/Core/Traits.h>

namespace orange
{
    template <typename... Types>
    class Variant
    {
    public:
        static constexpr size_t MaxElementSize = Max({sizeof(Types)...});
        static constexpr size_t Alignment      = Max({alignof(Types)...});
        
        explicit Variant()
        {
        }

        template <typename T, typename... Args>
        Variant(Args&&... args)
        {
            Construct<T>(Forward(args)...);
        }

        template <typename T, typename... Args>
        void Construct(Args&&... args)
        {
            new (Ptr<T>()) T(Forward(args)...);
        }

        template <typename T>       T* Ptr()       { return reinterpret_cast<T*>      (data.data); }
        template <typename T> const T* Ptr() const { return reinterpret_cast<const T*>(data.data); }

        template <typename T>       T& Get()       { return *Ptr<T>(); }
        template <typename T> const T& Get() const { return *Ptr<T>(); }
    private:
        AlignedStorage<MaxElementSize, Alignment> data;
    };
}
